'*******************************************************************************
'* KLS - KirchenLichtSteuerung                                                 *
'* Version 2.0.0                                                               *
'* Letzte nderung: 21.10.2023                                                 *
'*                                                                             *
'* Dr.-Ing. Christian Nding                                                   *
'* Infos unter https://www.pcdimmer.de                                         *
'*                                                                             *
'* Target: Atmel ATmega32 @ 16MHz                                              *
'*******************************************************************************
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version2 of
' the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
' General Public License for more details.
'
' If you have no copy of the GNU General Public License, write to the
' Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
'*******************************************************************************

$regfile = "m32def.dat"
$crystal = 16000000
$hwstack = 96
$swstack = 128
$framesize = 288
$map



' ******************************************************************************
' * Funktionsdeklarationen                                                     *
' ******************************************************************************

' Subs und Funktionen
Declare Sub Refreshscreen
Declare Sub Performkeypress(byval Key As Byte)

Declare Sub Calcsunrise(day As Byte , Month As Byte)
Declare Sub Calcsunset(day As Byte , Month As Byte)
Declare Function Calcadvent(year As Byte) As Word
Declare Function Tosysday(byval Day As Byte , Byval Month As Byte , Byval Year As Byte) As Word
'$lib "Eastern.lib"
Declare Function Eastern(year As Byte) As Word
'$external Eastern

Declare Sub Enlightchurch
Declare Sub Startscene(byval Scene As Byte)
Declare Sub Savescene(scene As Byte)

Declare Sub Setdmxch(byval Channel As Word , Byval Startvalue As Byte , Byval Endvalue As Byte , Byval Fadetime As Word)
Declare Sub Stepupch(byval Channel As Word)
Declare Sub Stepdownch(byval Channel As Word)



' ******************************************************************************
' * Peripheriedeklarationen                                                    *
' ******************************************************************************

' Zeitformat definieren
Config Date = Dmy , Separator = .                           '_day , _month, _year , _sec, _hour, _min

' Watchdog konfigurieren
Config Watchdog = 2048

' RS232-Port 1 einrichten (2 Stoppbits, 8 Datenbits und 250 kBaud)
Config Com1 = 250000 , Synchrone = 0 , Parity = None , Stopbits = 2 , Databits = 8 , Clockpol = 0

' LCD einrichten
Config Lcdpin = Pin , Db4 = Portc.0 , Db5 = Portc.1 , Db6 = Portc.2 , Db7 = Portc.3 , E = Portc.4 , Rs = Portc.5
Config Lcd = 16 * 2

' DCF77 Empfang einrichten
Config Portc.7 = Input
Config Dcf77 = Pinc.7 , Timer = 1 , Timer1sec = 0 , Debug = 0 , Check = 2 , Update = 0 , Gosub = Sectic
Portc.7 = 0                                                 ' Pullups am Empfangspin fr DCF77 Empfang ausschalten

' PWM-Dimmer fr Display einrichten
'Config Timer0 = Pwm , Pwm = On , Prescale = 1 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down
'Enable Timer0
'Start Timer0
'Ocr0a = 255

' Dimmerkernel einrichten
Config Timer2 = Timer , Prescale = 128 , Pwm = Off          '1 | 8 | 32 | 64 | 128 | 256 | 1024
On Timer2 Dimmerkernel
Timer2 = 130                                                ' Timer-Interrupt jede ms ausfhren (16000000MHz/128/125=1000kHz)
Enable Timer2
' DMX512-Interrupt konfigurieren
On Utxc Dmxsend                                             'ISR fr DMX512-Transmitter
Enable Utxc

' Tastenentprellung einrichten
'Config Debounce = 30                                        'Standard-Debounce-Zeit ist 25ms. Wir verwenden hier aber 30ms



' ******************************************************************************
' * Variablendeklarationen                                                     *
' ******************************************************************************

' Allgemeine Variablendeklarationen
Dim B_global As Byte
Dim W_global As Word
Dim W2_global As Word
Dim W3_global As Word

' Variablen fr Interrupt
Dim Bi As Byte

' Variablendeklaration fr Zeitsteuerung
Dim _sunrise(12) As Eram Word At 1                          ' Byte 1 bis 24   !WARNING! Byte 0 is bad designed in AVRs and must not be used
Dim _sunset(12) As Eram Word At 24 + 1                      ' Byte 25 bis 48
Dim _setup(8) As Eram Byte At 48 + 1                        ' Byte 49 bis 56
                                                              ' Bytes 57 bis xxx sind fr Szenenspeicher reserviert

Dim Summertime As Bit
Dim Sunrisehour As Byte
Dim Sunriseminute As Byte
Dim Sunrise As Word
Dim Sunsethour As Byte
Dim Sunsetminute As Byte
Dim Sunset As Word
Dim Manualhour_turnoff As Byte
Dim Manualminute_turnoff As Byte
Dim Currenttimeminute As Word

Dim Schedulerdone_sunset As Bit
Dim Schedulerdone_sunrise As Bit
Dim Schedulerdone_newyear As Bit
Dim Switchofftriac As Bit

' Variablendeklaration fr DCF77-Empfnger
Dim Weekday As Byte                                         ' Globaler Wochentag
Dim Sectick As Bit
Dim Seconds As Word

' Variablendeklaration fr DMX512-Ausgabe
Const Lastchan = 56
Const Lastdevice = 9
Const Channelperdevice = 6
Const Lastscene = 20
Dim Dmxfield(lastchan) As Byte
Dim Newdmxfield(lastchan) As Byte
Dim Dmxsteps(lastchan) As Word
Dim Curdmxsteps(lastchan) As Word
Dim Curdmxch As Word
Dim Dmxstate As Byte
Dim Globalfadetime As Byte
Dim Currentdevice As Byte
Dim Masterfader As Byte
Dim Strobofader As Byte

' Variablendeklaration fr Display
Dim Screenmode As Byte
Dim Screenoption As Byte



' ******************************************************************************
' * Bootsequenz der Steuerung                                                  *
' ******************************************************************************

Upperline
Lcd "  KLS365 v2.00  "
Lowerline
Lcd "-= 21.10.2023 =-"

Dmxstate = 0                                                '0=Break, 1=Startb, 2=Data
Curdmxch = 0
Currentdevice = 1
Globalfadetime = 60
Seconds = 0
Screenmode = 1
Screenoption = 1
Masterfader = 255
Strobofader = 0

' Uhr voreinstellen (auf einen Wert, an dem garantiert kein Ereignis stattfindet. Am 01.01. um 00:00 wrde nmlich z.B. Stroboeffekt starten)
Date$ = "02.02.23"
Time$ = "08:00:00"

' Variablen aus dem EEPROM lesen
Manualhour_turnoff = _setup(1)
Manualminute_turnoff = _setup(2)
Globalfadetime = _setup(3)

'Uart einrichten
Ucsrc = &B10001110                                          '8N2

' Triac-Ausgang (Ausgang ist Invertiert: Set=AUS Reset=EIN
Config Portd.7 = Output
Triac Alias Portd.7
Set Triac                                                   'Triac ausschalten
Reset Switchofftriac

' Pullups fr Taster aktivieren
Config Portb.4 = Input
Config Portb.5 = Input
Config Portb.6 = Input
Config Portb.7 = Input
Portb.4 = 1                                                 ' Pullups fr Taster einschalten
Portb.5 = 1                                                 ' Pullups fr Taster einschalten
Portb.6 = 1                                                 ' Pullups fr Taster einschalten
Portb.7 = 1                                                 ' Pullups fr Taster einschalten
Key1 Alias Pinb.6
Key2 Alias Pinb.7
Key3 Alias Pinb.4
Key4 Alias Pinb.5

' Hintergrundbeleuchtung
Config Portc.6 = Output
Backlight Alias Portc.6
Set Backlight                                               ' Hintergrundbeleuchtung einschalten

'LEDs
Config Portb.0 = Output
Led_dcf Alias Portb.0
Config Portb.1 = Output
Led_gb Alias Portb.1
Config Portb.2 = Output
Led_gn Alias Portb.2

Set Led_dcf
Reset Led_gb
Reset Led_gn
Wait 1

Reset Led_dcf
Set Led_gb
Reset Led_gn
Wait 1

Reset Led_dcf
Reset Led_gb
Set Led_gn
Wait 1

Enable Interrupts

'DMX512 starten
Udr = 0                                                     ' Einmalige Ausgabe einer Null, um den ersten "Transmission Complete" Interrupt zu generieren

Upperline
Lcd "Christian N" ; Chr(239) ; "ding"
Lowerline
Lcd "Tel:0179/5014852"                                      ' wer den Code hier sieht und Fragen hat, darf mich auch gerne anrufen ;-)

Wait 5

Start Watchdog                                              ' Wachhund starten

' Zeiten fr Sonnenauf- und -untergang berechnen und Sunrise-Scheduler zunchst auf "Done" stellen, da bei Start 08:00 eingestellt ist
Call Calcsunrise(_day , _month)
Call Calcsunset(_day , _month)
Set Schedulerdone_sunrise
Reset Schedulerdone_sunset

' Dimmer auf 100% und Strobo auf 0% setzen
For W_global = 1 To Lastdevice
   W2_global = W_global * Channelperdevice
   Call Setdmxch(w2_global , Dmxfield(w2_global) , Strobofader , 0)       ' Strobo auf 0%
   If W_global = Lastdevice Then
      W2_global = W2_global - 5
   Else
      W2_global = W2_global - 1
   End If
   Call Setdmxch(w2_global , Dmxfield(w2_global) , Masterfader , 0)       ' Dimmer auf 100%
Next


' ******************************************************************************
' * Hauptschleife                                                              *
' ******************************************************************************

Reset Led_dcf
Reset Led_gb
Set Led_gn

Main:
Do
   Reset Watchdog

   ' ***************************************************************************
   ' * Folgender Code wird jede Sekunde ausgefhrt                             *
   ' ***************************************************************************
   If Sectick = 1 Then
      Reset Sectick
      Toggle Led_gb

      Call Refreshscreen
      Reset Watchdog

      ' Minutenbasierten Einheitszeitwert berechnen
      Currenttimeminute = _hour * 60
      Currenttimeminute = Currenttimeminute + _min
      If Summertime = 0 Then
         ' Winterzeitversatz korrigieren (Currenttimeminute=Currenttimeminute+30)
         Currenttimeminute = Currenttimeminute + 30         ' Vergleichszeit um 30 Minuten nach hinten verschieben, damit Licht etwas frher angeht
      End If

      Reset Watchdog

      ' Prfen, ob Sonnenuntergang erreicht
      If Currenttimeminute >= Sunset Or Currenttimeminute < Sunrise Then
         If Schedulerdone_sunset = 0 Then
            Stop Watchdog

            Set Schedulerdone_sunset
            Call Enlightchurch                              ' Triac einschalten und Kirchenbeleuchtung einblenden

            Start Watchdog
         End If
      Else
         Reset Schedulerdone_sunset
      End If

      ' Prfen, ob Sonnenaufgang erreicht
      If Currenttimeminute >= Sunrise And Currenttimeminute < Sunset Then
         If Schedulerdone_sunrise = 0 Then
            Stop Watchdog

            Set Schedulerdone_sunrise
            Call Startscene(0)                              ' Kirchenbeleuchtung ausblenden und Triac ausschalten

            Start Watchdog
         End If
      Else
         Reset Schedulerdone_sunrise
      End If

      ' Prfen, ob Neujahr
      If _month = 1 And _day = 1 And _hour = 0 And _min = 0 Then
         If Schedulerdone_newyear = 0 Then
            Stop Watchdog

            ' Frohes Neues Jahr
            Set Schedulerdone_newyear                       ' sicherstellen, dass dieser If-Block nur einmal aufgerufen wird
            Seconds = 180                                   ' 5-Minuten-Counter auf 2 Minuten Restlaufzeit stellen (also ab jetzt 2-Minuten Blinken/Blitzen)

            ' Strobokanle der Lampen auf Strobe schalten
            For W_global = 1 To Lastdevice
               W2_global = W_global * Channelperdevice
               B_global = Rnd(30)
               B_global = B_global + 200
               Call Setdmxch(w2_global , Dmxfield(w2_global) , B_global , 0)       ' Strobo mit Zufallswerten zwischen 200 und 230 (mittleres Strobe)
            Next

            Start Watchdog
         End If
      End If

      ' ************************************************************************
      ' * Folgender Code wird alle 5 Minuten ausgefhrt                    *
      ' ************************************************************************
      ' ([Seconds]=Word=max. 299, also 300s/60=5min)
      Incr Seconds
      If Seconds > 299 Then
         Seconds = 0
         Stop Watchdog

         Reset Schedulerdone_newyear
         Screenmode = 1                                     ' wieder auf Men 1 schalten
         Screenoption = 1                                   ' Menoptionen wieder auf 1 schalten

         If Switchofftriac = 1 Then
            ' Triac ausschalten
            Reset Switchofftriac
            Set Triac
         End If

         ' Sonnenauf- und Sonnenuntergang berechnen
         Upperline
         Lcd "Berechne        "
         Lowerline
         Lcd "Sonnenzeiten... "
         Call Calcsunrise(_day , _month)
         Call Calcsunset(_day , _month)

         ' Dimmer auf 100% und Strobo auf 0% setzen
         Upperline
         Lcd "Setze Dimmer und"
         Lowerline
         Lcd "Strobe Kanal... "
         For W_global = 1 To Lastdevice
            W2_global = W_global * Channelperdevice
            Call Setdmxch(w2_global , Dmxfield(w2_global) , Strobofader , 0)       ' Strobo auf 0%
            If W_global = Lastdevice Then
               W2_global = W2_global - 5
            Else
               W2_global = W2_global - 1
            End If
            Call Setdmxch(w2_global , Dmxfield(w2_global) , Masterfader , 0)       ' Dimmer auf 100%
         Next

         Reset Backlight                                    ' LCD Hintergrundbeleuchtung ausschalten
         Start Watchdog
      End If
      ' ************************************************************************
      ' * Ende des 5-Minuten-Timers                                        *
      ' ************************************************************************
      ' ************************************************************************
      ' * Folgender Code wird jede Minute ausgefhrt                           *
      ' ************************************************************************
'      If Seconds = 0 or Seconds = 60 or Seconds = 120 or Seconds = 180 or Seconds=240 Then
         ' Softclock nach DCF77-Zeit stellen
'         Time$ = Time(dcf_sec)
'         Date$ = Date(dcf_day)
'      End If
      ' ************************************************************************
      ' * Ende des Minuten-Timers                                              *
      ' ************************************************************************

      Reset Watchdog

      ' DCF77-Statusanzeige
      Select Case Dcf77timezone()
         Case 0 : Toggle Led_dcf                            ' kein gltiges DCF77-Signal
         Case 1 : Reset Led_dcf                             ' Gltiges DCF77-Signal whrend Winterzeit
         Case 2 : Set Led_dcf                               ' Gltiges DCF77-Signal whrend Sommerzeit
      End Select
   End If
   ' ***************************************************************************
   ' * Ende des Sekunden-Timers                                                *
   ' ***************************************************************************


   ' ***************************************************************************
   ' * Tasteten abfragen und entsprechend verarbeiten                          *
   ' ***************************************************************************

'   Debounce Pinb.6 , 0 , Keysub1 , Sub
'   Debounce Pinb.7 , 0 , Keysub2 , Sub
'   Debounce Pinb.4 , 0 , Keysub3 , Sub
'   Debounce Pinb.5 , 0 , Keysub4 , Sub

   If Key1 = 0 Then
      Stop Watchdog                                         ' Wachhund stoppen
      Gosub Keysub1
      Start Watchdog                                        ' Wachhund wieder starten
   End If
   If Key2 = 0 Then
      Stop Watchdog                                         ' Wachhund stoppen
      Gosub Keysub2
      Start Watchdog                                        ' Wachhund wieder starten
   End If
   If Key3 = 0 Then
      Stop Watchdog                                         ' Wachhund stoppen
      Gosub Keysub3
      Start Watchdog                                        ' Wachhund wieder starten
   End If
   If Key4 = 0 Then
      Stop Watchdog                                         ' Wachhund stoppen
      Gosub Keysub4
      Start Watchdog                                        ' Wachhund wieder starten
   End If

   ' ***************************************************************************
   ' * Ende des Tastenabfragens                                                *
   ' ***************************************************************************
Loop
End



' ******************************************************************************
' * Tasteneingabefunktionen                                                    *
' ******************************************************************************

Keysub1:
   ' Mentaste gedrck
   Call Performkeypress(1)
   Call Refreshscreen
   Waitms 100
Return

Keysub2:
   'Sel/OK gedrckt
   Call Performkeypress(2)
   Call Refreshscreen
   Waitms 100
Return

Keysub3:
   '+ gedrckt
   While Key3 = 0
      Call Performkeypress(3)
      If Key2 = 0 Then
         Waitms 5
      Else
         Waitms 50
         Call Refreshscreen
      End If
   Wend
   Call Refreshscreen
Return

Keysub4:
   '- gedrck
   While Key4 = 0
      Call Performkeypress(4)
      If Key2 = 0 Then
         Waitms 5
      Else
         Waitms 50
         Call Refreshscreen
      End If
   Wend
   Call Refreshscreen
Return



' ******************************************************************************
' * Programmfunktionen                                                         *
' ******************************************************************************

Sub Refreshscreen
   Local W As Word
   Local Triac_display As Byte

   If Triac = 1 Then
      Triac_display = 0
   Else
      Triac_display = 1
   End If

      Select Case Screenmode
         Case 1:
            ' Standardanzeige
            Cursor Off
            Upperline
            If Dcf_status.7 = 1 Then
               Weekday = Dayofweek() + 1                    ' Wochentag errechnen
               Lcd Lookupstr(weekday , Wochentag) ; "," ; Time$       ' Wochentag, Zeit und Datum anzeigen
            Else
               Lcd "D?," ; Time$                            ' Wochentag, Zeit und Datum anzeigen
            End If
            Locate 1 , 9
            Lcd " E:" ; Sunsethour ; ":" ; Sunsetminute ; "  "
            Lowerline
            Lcd Date$ ; " A:" ; Sunrisehour ; ":" ; Sunriseminute ; "   "
         Case 2:
            ' Kanalwertnderungen
            Upperline
            Lcd "Lampe:" ; Currentdevice ; "         "
            Locate 1 , 13
            Lcd "M" ; Hex(Masterfader) ; "   "
            Lowerline
            W = Currentdevice * Channelperdevice
            If Currentdevice = Lastdevice Then
               W = W - 4
            Else
               W = W - 5
            End If
            Locate 2 , 1
            Lcd "R" ; Hex(Dmxfield(w)) ; "   "
            W = W + 1
            Locate 2 , 5
            Lcd "G" ; Hex(Dmxfield(w)) ; "   "
            W = W + 1
            Locate 2 , 9
            Lcd "B" ; Hex(Dmxfield(w)) ; "   "
            W = W + 1
            Locate 2 , 13
            Lcd "W" ; Hex(Dmxfield(w)) ; "   "

            Cursor On Blink
            Select Case Screenoption
               Case 1 :
                  ' Einstellung der aktuellen Lampe
                  Locate 1 , 7
               Case 2 :
                  ' Einstellung von Kanal R
                  Locate 2 , 2
               Case 3 :
                  ' Einstellung von Kanal G
                  Locate 2 , 6
               Case 4 :
                  ' Einstellung von Kanal B
                  Locate 2 , 10
               Case 5 :
                  ' Einstellung von Kanal W
                  Locate 2 , 14
               Case 6 :
                  ' Einstellung des Masterfaders
                  Locate 1 , 14
            End Select
         Case 3:
            ' Speichern von Szenen
            Cursor On Blink
            Upperline
            Lcd "Speichern unter "
            Lowerline
            Lcd "Szene <" ; Screenoption ; ">       "
            Locate 2 , 8
         Case 4:
            ' Speichern von Szenen
            Cursor On Blink
            Upperline
            Lcd "Aufrufen von    "
            Lowerline
            Lcd "Szene <" ; Screenoption ; ">       "
            Locate 2, 8
         Case 5:
            ' Einstellungen
            Cursor Off
            Upperline
            Lcd "AusZeit Fade Rel"

            Select Case Screenoption
               Case 1:
                  Lowerline
                  Lcd "<" ; Manualhour_turnoff ; ">:" ; Manualminute_turnoff ; "    "
                  Locate 2 , 9
                  Lcd Globalfadetime ; "     "
                  Locate 2 , 14
                  Lcd Triac_display ; "  "
               Case 2:
                  Lowerline
                  Lcd Manualhour_turnoff ; ":<" ; Manualminute_turnoff ; ">    "
                  Locate 2 , 9
                  Lcd Globalfadetime ; "     "
                  Locate 2 , 14
                  Lcd Triac_display ; "  "
               Case 3:
                  Lowerline
                  Lcd Manualhour_turnoff ; ":" ; Manualminute_turnoff ; "    "
                  Locate 2 , 8
                  Lcd "<" ; Globalfadetime ; ">  "
                  Locate 2 , 14
                  Lcd Triac_display ; "  "
               Case 4:
                  Lowerline
                  Lcd Manualhour_turnoff ; ":" ; Manualminute_turnoff ; "    "
                  Locate 2 , 9
                  Lcd Globalfadetime ; "     "
                  Locate 2 , 13
                  Lcd "<" ; Triac_display ; "> "
            End Select
      End Select
End Sub

Sub Performkeypress(byval Key As Byte)
   Local W As Word
   Local W2 As Word
      Set Backlight                                         ' Hintergrundlicht einschalten
      Seconds = 0                                           ' Verhindern dass Display ausgeht

      ' Men   1
      ' Sel/OK 2
      ' +      3
      ' -      4

      If Key = 1 Then
         ' Screenmode zwischen 1..5 umschalten
         If Screenmode = 5 Then
            Screenmode = 1
         Else
            Incr Screenmode
         End If
         Screenoption = 1
      End If

      Select Case Screenmode
         Case 1:
            ' Hauptbildschirm
         Case 2:
            ' Kanaleinstellungen
            ' Sel/OK Umschalten zwischen Kanlen
            ' +      Kanal/Lampe inkrementieren
            ' -      Kanal/Lampe dekrementieren
            Select Case Key
               Case 2:
                  If Screenoption = 5 Then
                     Screenoption = 1
                  Else
                     Incr Screenoption
                  End If
               Case 3:
                  Select Case Screenoption
                     Case 1:
                        If Currentdevice = Lastdevice Then
                           Currentdevice = 1
                        Else
                           Incr Currentdevice
                        End If
                     Case 2: ' Channel R
                        W = Currentdevice * Channelperdevice
                        If Currentdevice = Lastdevice Then
                           W = W - 4
                        Else
                           W = W - 5
                        End If
                        'If Dmxfield(w) < 255 Then
                           Newdmxfield(w) = Dmxfield(w) + 1
                           Dmxfield(w) = Newdmxfield(w)
                        'End If
                     Case 3: ' Channel G
                        W = Currentdevice * Channelperdevice
                        If Currentdevice = Lastdevice Then
                           W = W - 3
                        Else
                           W = W - 4
                        End If
                        'If Dmxfield(w) < 255 Then
                           Newdmxfield(w) = Dmxfield(w) + 1
                           Dmxfield(w) = Newdmxfield(w)
                        'End If
                     Case 4: ' Channel B
                        W = Currentdevice * Channelperdevice
                        If Currentdevice = Lastdevice Then
                           W = W - 2
                        Else
                           W = W - 3
                        End If
                        'If Dmxfield(w) < 255 Then
                           Newdmxfield(w) = Dmxfield(w) + 1
                           Dmxfield(w) = Newdmxfield(w)
                        'End If
                     Case 5: ' Channel W
                        W = Currentdevice * Channelperdevice
                        If Currentdevice < Lastdevice Then ' Lastdevice has no White-Channel
                           W = W - 2
                           'If Dmxfield(w) < 255 Then
                              Newdmxfield(w) = Dmxfield(w) + 1
                              Dmxfield(w) = Newdmxfield(w)
                           'End If
                        End If
                     Case 6: ' Channel Masterdimmer
                        'If Masterfader < 255 Then
                           Masterfader = Masterfader + 1
                        'End If

                        For W = 1 To Lastdevice
                           W2 = W * Channelperdevice
                           If W = Lastdevice Then
                              W2 = W2 - 5
                           Else
                              W2 = W2 - 1
                           End If
                           Call Setdmxch(w2 , Dmxfield(w2) , Masterfader , 0)       ' Dimmer
                        Next
                  End Select
               Case 4:
                  Select Case Screenoption
                     Case 1:
                        If Currentdevice = 1 Then
                           Currentdevice = Lastdevice
                        Else
                           Decr Currentdevice
                        End If
                     Case 2:
                        W = Currentdevice * Channelperdevice
                        If Currentdevice = Lastdevice Then
                           W = W - 4
                        Else
                           W = W - 5
                        End If
                        If Dmxfield(w) > 0 Then
                           Newdmxfield(w) = Dmxfield(w) - 1
                           Dmxfield(w) = Newdmxfield(w)
                        End If
                     Case 3:
                        W = Currentdevice * Channelperdevice
                        If Currentdevice = Lastdevice Then
                           W = W - 3
                        Else
                           W = W - 4
                        End If
                        If Dmxfield(w) > 0 Then
                           Newdmxfield(w) = Dmxfield(w) - 1
                           Dmxfield(w) = Newdmxfield(w)
                        End If
                     Case 4:
                        W = Currentdevice * Channelperdevice
                        If Currentdevice = Lastdevice Then
                           W = W - 2
                        Else
                           W = W - 3
                        End If
                        If Dmxfield(w) > 0 Then
                           Newdmxfield(w) = Dmxfield(w) - 1
                           Dmxfield(w) = Newdmxfield(w)
                        End If
                     Case 5:
                        W = Currentdevice * Channelperdevice
                        If Currentdevice < Lastdevice Then
                           W = W - 2
                           If Dmxfield(w) > 0 Then
                              Newdmxfield(w) = Dmxfield(w) - 1
                              Dmxfield(w) = Newdmxfield(w)
                           End If
                        End If
                     Case 6:
                        If Masterfader > 0 Then
                           Masterfader = Masterfader - 1
                        End If

                        For W = 1 To Lastdevice
                           W2 = W * Channelperdevice
                           If W = Lastdevice Then
                              W2 = W2 - 5
                           Else
                              W2 = W2 - 1
                           End If
                           Call Setdmxch(w2 , Dmxfield(w2) , Masterfader , 0)       ' Dimmer
                        Next
                  End Select
            End Select
         Case 3:
            ' Szenen speichern
            ' Sel/OK Szene abspeichern
            ' +      Szene inkrementieren
            ' -      Szene dekrementieren
            Select Case Key
               Case 2:
                  While Key2 = 0
                     If Key1 = 0 Then
                        Call Savescene(screenoption)
                        Upperline
                        Lcd "Szene " ; Screenoption ; " wurde   "
                        Lowerline
                        Lcd "gespeichert...  "
                        Wait 1
                     End If
                  Wend
               Case 3:
                  If Screenoption = Lastscene Then
                     Screenoption = 1
                  Else
                     Incr Screenoption
                  End If
               Case 4:
                  If Screenoption = 1 Then
                     Screenoption = Lastscene
                  Else
                     Decr Screenoption
                  End If
            End Select
         Case 4:
            ' Szene aufrufen
            ' Sel/OK Szene aufrufen
            ' +      Szene inkrementieren
            ' -      Szene dekrementieren
            Select Case Key
               Case 2:
                  Call Startscene(screenoption)
                  Wait 1
               Case 3:
                  If Screenoption = Lastscene Then
                     Screenoption = 0
                  Else
                     Incr Screenoption
                  End If
               Case 4:
                  If Screenoption = 0 Then
                     Screenoption = Lastscene
                  Else
                     Decr Screenoption
                  End If
            End Select
         Case 5:
            ' Generelle Einstellungen
            ' Sel/OK Optionsauswahl
            ' +      Wert inkrementieren
            ' -      Wert dekrementieren
            Select Case Key
               Case 2:
                  If Screenoption = 4 Then
                     Screenoption = 1
                  Else
                     Incr Screenoption
                  End If
                  While Key2 = 0
                     If Key1 = 0 Then
                        _setup(1) = Manualhour_turnoff
                        _setup(2) = Manualminute_turnoff
                        _setup(3) = Globalfadetime

                        Upperline
                        Lcd "Einstellungen   "
                        Lowerline
                        Lcd "gespeichert.    "
                        Wait 1
                     End If
                  Wend
               Case 3:
                  Select Case Screenoption
                     Case 1:
                        If Manualhour_turnoff = 12 Then
                           Manualhour_turnoff = 0
                        Else
                           Incr Manualhour_turnoff
                        End If
                        ' Ausschaltzeit neu berechnen
                        Call Calcsunrise(_day , _month)
                        Call Calcsunset(_day , _month)
                     Case 2:
                        If Manualminute_turnoff = 59 Then
                           Manualminute_turnoff = 0
                        Else
                           Incr Manualminute_turnoff
                        End If
                        ' Ausschaltzeit neu berechnen
                        Call Calcsunrise(_day , _month)
                        Call Calcsunset(_day , _month)
                     Case 3:
                        If Globalfadetime = 255 Then
                           Globalfadetime = 0
                        Else
                           Incr Globalfadetime
                        End If
                     Case 4:
                        Reset Triac                         ' Triac einschalten
                  End Select
               Case 4:
                  Select Case Screenoption
                     Case 1:
                        If Manualhour_turnoff = 0 Then
                           Manualhour_turnoff = 12
                        Else
                           Decr Manualhour_turnoff
                        End If
                        ' Ausschaltzeit neu berechnen
                        Call Calcsunrise(_day , _month)
                        Call Calcsunset(_day , _month)
                     Case 2:
                        If Manualminute_turnoff = 0 Then
                           Manualminute_turnoff = 59
                        Else
                           Decr Manualminute_turnoff
                        End If
                        ' Ausschaltzeit neu berechnen
                        Call Calcsunrise(_day , _month)
                        Call Calcsunset(_day , _month)
                     Case 3:
                        If Globalfadetime = 0 Then
                           Globalfadetime = 255
                        Else
                           Decr Globalfadetime
                        End If
                     Case 4:
                        Set Triac                           ' Triac ausschalten
                  End Select
            End Select
      End Select
End Sub

Sub Calcsunrise(day As Byte , Month As Byte)
   Local B As Byte
   Local S As Single

   Local Sunrisetime As Single
   Local Month_ptr As Single
   Local Valuetop As Word
   Local Valuebot As Word

   If Month < 1 Then
      B = 1
   Elseif Month > 12 Then
      B = 12
   Else
      B = Month
   End If

   ' Manuelle Zeit zuweisen (wird im nchsten Schritt ggfs. wieder berschrieben)
   Sunrisehour = Manualhour_turnoff
   Sunriseminute = Manualminute_turnoff

   If Manualhour_turnoff = 12 Then
      If Manualminute_turnoff = 0 Then
         ' Lineare Interpolation um bestmglichen Fit zu erhalten
         ' A) oberen und unteren Wert finden und laden
         If B = 12 Then
            Valuetop = _sunrise(1)
            Valuebot = _sunrise(b)
         Else
            B = B + 1
            Valuetop = _sunrise(b)
            B = B - 1
            Valuebot = _sunrise(b)
         End If
         ' B) Monatspointer berechnen (luft von 0.00 bis ca. 1)
         Month_ptr = Day / 30                               ' etwas ungenau, da manche Monate 28, 29 oder 31 Tage haben, aber es sollte genau genug sein
         ' C) Sunrisetime interpolieren
         Sunrisetime = Valuetop - Valuebot                  ' TODO: WORD IST ALS DATENTYP UNGEEIGNET!!!
         Sunrisetime = Sunrisetime * Month_ptr
         Sunrisetime = Sunrisetime + Valuebot

         ' Nun Sonnenaufgangszeit getrennt nach Stunde und Minute berechnen
         ' Stunde berechnen
         S = Sunrisetime / 1000                             ' S=17913/1000=17,913  Beispielwerte fr Sunrisetime=17913
         Sunrisehour = Int(s)                               ' Sunrisehour=17
         S = S - Sunrisehour                                ' S=17,913 - 17=0,913

         ' ggfs. Sommerzeit einberechnen. Dcf77timezone() ist 0, wenn kein gltiges DCF77-Signal vorliegt. In diesem Fall wird zur Sicherheit nichts an der bisher erkannten Sommerzeiteinstellung gendert
         If Dcf77timezone() = 1 Then
            Reset Summertime
         Elseif Dcf77timezone() = 2 Then
            Set Summertime
         End If

         If Summertime = 1 Then
            Sunrisehour = Sunrisehour + 1                   ' whrend Sommerzeit 1 Stunde aufrechnen: Sunrisehour=18
         End If

         S = S * 60                                         'S=0,913*60=54,78
         Sunriseminute = Round(s)                           'Sunriseminute=55
      End If
   End If

   Sunrise = Sunrisehour * 60
   Sunrise = Sunrise + Sunriseminute
End Sub

Sub Calcsunset(day As Byte , Month As Byte)
   Local B As Byte
   Local W As Word
   Local S As Single

   Local Sunsettime As Single
   Local Month_ptr As Single
   Local Valuetop As Word
   Local Valuebot As Word

   If Month < 1 Then
      B = 1
   Elseif Month > 12 Then
      B = 12
   Else
      B = Month
   End If

   ' Lineare Interpolation um bestmglichen Fit zu erhalten
   ' A) oberen und unteren Wert finden und laden
   If B = 12 Then
      Valuetop = _sunset(1)
      Valuebot = _sunset(b)
   Else
      B = B + 1
      Valuetop = _sunset(b)
      B = B - 1
      Valuebot = _sunset(b)
   End If
   ' B) Monatspointer berechnen (luft von 0.00 bis ca. 1)
   Month_ptr = Day / 30                                     ' etwas ungenau, da manche Monate 28, 29 oder 31 Tage haben, aber es sollte genau genug sein
   ' C) Sunrisetime interpolieren
   Sunsettime = Valuetop - Valuebot
   Sunsettime = Sunsettime * Month_ptr
   Sunsettime = Sunsettime + Valuebot

   ' Nun Sonnenuntergangszeit getrennt nach Stunde und Minute berechnen
   ' Stunde berechnen
   S = Sunsettime / 1000                                    ' S=17913/1000=17,913  Beispielwerte fr Sunsettime=17913

   Sunsethour = Int(s)                                      ' Sunsethour=17
   S = S - Sunsethour                                       ' S=17,913 - 17=0,913

   ' ggfs. Sommerzeit einberechnen. Dcf77timezone() ist 0, wenn kein gltiges DCF77-Signal vorliegt. In diesem Fall wird zur Sicherheit nichts an der bisher erkannten Sommerzeiteinstellung gendert
   If Dcf77timezone() = 1 Then
      Reset Summertime
   Elseif Dcf77timezone() = 2 Then
      Set Summertime
   End If

   If Summertime = 1 Then
      Sunsethour = Sunsethour + 1                           ' whrend Sommerzeit 1 Stunde aufrechnen: Sunsethour=18
   End If

   S = S * 60                                               'S=0,913*60=54,78
   Sunsetminute = Round(s)                                  'Sunsetminute=55

   Sunset = Sunsethour * 60
   Sunset = Sunset + Sunsetminute
End Sub

Function Calcadvent(year As Byte) As Word
   Local W As Word
   Local W2 As Word
   Local B As Byte

   W = Tosysday(25 , 12 , Year)
   B = Dayofweek(w) + 1

   W2 = W - B

   Calcadvent = W2 - 21
End Function

Function Tosysday(byval Day As Byte , Byval Month As Byte , Byval Year As Byte) As Word
   Local Bday As Byte , Bmonth As Byte , Byear As Byte

   Bday = Day
   Bmonth = Month
   Byear = Year

   Tosysday = Sysday(bday)
End Function

Function Eastern(year As Byte) As Word
   Local Helper As Integer
   Local B As Byte

   Local X As Integer
   Local K As Integer
   Local S As Integer
   Local M As Integer
   Local A As Integer
   Local D As Integer
   Local R As Integer
   Local Og As Integer
   Local Sz As Integer
   Local Oe As Integer

   Local Os As Integer
   Local Result As Word

   'Anwendung der Gauschen Osterformel
   X = Year + 2000

   K = X \ 100

   S = 3 * K
   S = S + 3
   S = S \ 4
   S = 2 - S

   M = 3 * K
   M = M + 3
   M = M \ 4
   M = M + 15
   Helper = 8 * K
   Helper = Helper + 13
   Helper = Helper \ 25
   M = M - Helper

   A = X Mod 19

   D = 19 * A
   D = D + M
   D = D Mod 30

   R = A \ 11
   R = R + D
   R = R \ 29

   Og = 21 + D
   Og = Og - R

   Sz = X \ 4
   Sz = Sz + S
   Sz = Sz + X
   Sz = Sz Mod 7
   Sz = 7 - Sz

   Oe = Og - Sz
   Oe = Oe Mod 7
   Oe = 7 - Oe

   Os = Og + Oe

   ' Unterscheidung nach Mrz oder April
   If Os > 31 Then
      B = Os - 31
      Result = Tosysday(b , 4 , Year)
   Else
      B = Os
      Result = Tosysday(b , 3 , Year)
   End If

   Eastern = Result
End Function

Sub Enlightchurch
   Local Now As Word
   Local B As Byte

   Local Specialday_rosenmontag As Word
   Local Specialday_gruendonnerstag As Word
   Local Specialday_karfreitag As Word
   Local Specialday_karsamstag As Word
   Local Specialday_ostersonntag As Word
   Local Specialday_ostermontag As Word
   Local Specialday_himmelfahrt As Word
   Local Specialday_pfingstsonntag As Word
   Local Specialday_pfingstmontag As Word
   Local Specialday_fronleichnam As Word
   Local Specialday_reformationstag As Word
   Local Specialday_volkstrauertag As Word
   Local Specialday_bussbettag As Word
   Local Specialday_ewigkeitssonntag As Word
   Local Specialday_advent1 As Word
   Local Specialday_advent2 As Word
   Local Specialday_advent3 As Word
   Local Specialday_advent4 As Word
   Local Specialday_weihnachten As Word
   Local Specialday_weihnachten1 As Word
   Local Specialday_weihnachten2 As Word
   Local Specialday_silvester As Word

   ' Aktuelles Datum berechnen
   Now = Tosysday(_day , _month , _year)

   ' Feiertage um Ostern herum berechnen
   Specialday_ostersonntag = Eastern(_year)

   Specialday_rosenmontag = Specialday_ostersonntag - 48
   Specialday_karfreitag = Specialday_ostersonntag - 2
   Specialday_karsamstag = Specialday_ostersonntag - 1
   Specialday_gruendonnerstag = Specialday_ostersonntag - 3
   Specialday_ostermontag = Specialday_ostersonntag + 1
   Specialday_himmelfahrt = Specialday_ostersonntag + 39
   Specialday_pfingstsonntag = Specialday_ostersonntag + 49
   Specialday_pfingstmontag = Specialday_ostersonntag + 50
   Specialday_fronleichnam = Specialday_ostersonntag + 60

   ' Feiertage um Advent herum berechnen
   Specialday_advent1 = Calcadvent(_year)
   Specialday_advent2 = Specialday_advent1 + 7
   Specialday_advent3 = Specialday_advent2 + 7
   Specialday_advent4 = Specialday_advent3 + 7
   Specialday_reformationstag = Tosysday(31 , 10 , _year)
   Specialday_volkstrauertag = Specialday_advent1 - 14
   Specialday_bussbettag = Specialday_advent1 - 11
   Specialday_ewigkeitssonntag = Specialday_advent1 - 7
   Specialday_weihnachten = Tosysday(24 , 12 , _year)
   Specialday_weihnachten1 = Specialday_weihnachten + 1
   Specialday_weihnachten2 = Specialday_weihnachten + 2
   Specialday_silvester = Specialday_weihnachten + 7

   Select Case Now
      Case Specialday_rosenmontag : Call Startscene(15)     'Rosenmontag       Farbe: Bunt

      Case Specialday_gruendonnerstag : Call Startscene(12) 'Grndonnerstag    Farbe: Grn/Wei
      Case Specialday_karfreitag : Call Startscene(6)       'Karfreitag        Farbe: Schwarz/Rot
      Case Specialday_karsamstag : Call Startscene(7)       'Karsamstag        Farbe: Schwarz/Violett
      Case Specialday_ostersonntag : Call Startscene(8)     'Ostersonntag      Farbe: Wei
      Case Specialday_ostermontag : Call Startscene(8)      'Ostermontag       Farbe: Wei

      Case Specialday_himmelfahrt : Call Startscene(8)      'Himmelfahrt       Farbe: Wei
      Case Specialday_pfingstsonntag : Call Startscene(9)   'Pfingstsonntag    Farbe: Rot
      Case Specialday_pfingstmontag : Call Startscene(9)    'Pfingstmontag     Farbe: Rot
      Case Specialday_fronleichnam : Call Startscene(8)     'Fonleichnam       Farbe: Wei

      Case Specialday_reformationstag : Call Startscene(9)  'Reformationstag   Farbe: Rot
      Case Specialday_volkstrauertag : Call Startscene(10)  'Volkstrauertag    Farbe: Blau
      Case Specialday_bussbettag : Call Startscene(11)      'Bu- und Bettag   Farbe: Violett
      Case Specialday_ewigkeitssonntag : Call Startscene(12)       'Ewigk.son. Farbe: Grn/Wei

      Case Specialday_advent1 : Call Startscene(13)         '1. Advent         Farbe: Violett
      Case Specialday_advent2 : Call Startscene(13)         '2. Advent         Farbe: Violett
      Case Specialday_advent3 : Call Startscene(13)         '3. Advent         Farbe: Violett
      Case Specialday_advent4 : Call Startscene(13)         '4. Advent         Farbe: Violett
      Case Specialday_weihnachten : Call Startscene(14)     '24.12.            Farbe: Rot/Wei
      Case Specialday_weihnachten1 : Call Startscene(14)    '25.12.            Farbe: Rot/Wei
      Case Specialday_weihnachten2 : Call Startscene(14)    '26.12.            Farbe: Rot/Wei
      Case Specialday_silvester : Call Startscene(15)       '31.12.            Farbe: Bunt
      Case Else :
         B = Rnd(5)
         B = B + 1
         Call Startscene(b)                                 ' Jeden Abend zufllig eine von 5 Szenen darstellen
   End Select

   Call Refreshscreen
End Sub

Sub Startscene(byval Scene As Byte)
   Local W As Word
   Local W2 As Word
   Local Localfadetime As Word
   Local Address_ptr As Word
   Local Cr As Byte
   Local Cg As Byte
   Local Cb As Byte
   Local Cw As Byte

   If Scene = 0 Then
      ' Triac ausschalten
      Seconds = 0                                           ' Sorgt dafr, dass der Triac erst nach 5 Minuten ausgeschaltet wird
      Set Switchofftriac

      ' Alles ausblenden
      For W = 1 To Lastdevice
         ' DMX Kanal berechnen
         W2 = W * Channelperdevice
         If W = Lastdevice Then
            W2 = W2 - 4
         Else
            W2 = W2 - 5
         End If

         ' R, G, B und W ausblenden
         Localfadetime = Globalfadetime
         Call Setdmxch(w2 , Dmxfield(w2) , 0 , Localfadetime)       ' R
         W2 = W2 + 1
         Call Setdmxch(w2 , Dmxfield(w2) , 0 , Localfadetime)       ' G
         W2 = W2 + 1
         Call Setdmxch(w2 , Dmxfield(w2) , 0 , Localfadetime)       ' B
         If W < Lastdevice Then
            W2 = W2 + 1
            Call Setdmxch(w2 , Dmxfield(w2) , 0 , Localfadetime)    ' W
         End If
      Next

      Set Backlight
      Upperline
      Lcd "Beleuchtung wird"
      Lowerline
      Lcd "ausgeschaltet..."
      Wait 1
   Else
      ' Triac einschalten
      Reset Switchofftriac
      Reset Triac

      ' Szenen aus EEPROM lesen und starten
      For W = 1 To Lastdevice
         ' zunchst Speicheradresse berechnen
         ' Szenenweises weiterspringen
         W2 = Scene - 1                                     'W2=4-1=3
         W2 = W2 * Lastdevice                           'W2=Aktuelle Szene * Geraetezahl * Kanalspeicher = 3 * 8 * 4=96
         W2 = W2 * 4
         Address_ptr = 57 + W2                              'Address_ptr=57+96=153
         ' Gerteweises weiterspringen
         W2 = W - 1                                         'W2=1..9 - 1 = 0..8
         W2 = W2 * 4  ' abzuspeichernde Kanalzahl (R,G,B,W=4) 'W2=0..8 * 3=0..24
         Address_ptr = Address_ptr + W2                     'Address_ptr=138+0..24=138..162

         ' Werte aus EEPROM lesen
         Readeeprom Cr , Address_ptr                        ' Rot lesen
         Address_ptr = Address_ptr + 1                      ' Adresspointer fr Grn um 1 erhhen
         Readeeprom Cg , Address_ptr                        ' Grn lesen
         Address_ptr = Address_ptr + 1                      ' Adresspointer fr Blau um 1 erhhen
         Readeeprom Cb , Address_ptr                        ' Blau lesen
         Address_ptr = Address_ptr + 1                      ' Adresspointer fr Weiss um 1 erhhen
         Readeeprom Cw , Address_ptr                        ' Weiss lesen

         ' DMX Kanal berechnen
         W2 = W * Channelperdevice
         If W = Lastdevice Then
            W2 = W2 - 4
         Else
            W2 = W2 - 5
         End If

         ' R, G und B auf EEPROM-Wert blenden
         Localfadetime = Globalfadetime
         Call Setdmxch(w2 , Dmxfield(w2) , Cr , Localfadetime)       ' R
         W2 = W2 + 1
         Call Setdmxch(w2 , Dmxfield(w2) , Cg , Localfadetime)       ' G
         W2 = W2 + 1
         Call Setdmxch(w2 , Dmxfield(w2) , Cb , Localfadetime)       ' B
         If W < Lastdevice Then
            W2 = W2 + 1
            Call Setdmxch(w2 , Dmxfield(w2) , Cw , Localfadetime)       ' W
         End If
      Next

      Set Backlight
      Upperline
      Lcd "Szene " ; Scene ; " wird    "
      Lowerline
      Lcd "eingeblendet... "
      Wait 1
   End If
End Sub

Sub Savescene(scene As Byte)
   Local W As Word
   Local W2 As Word
   Local Address_ptr As Word

   For W = 1 To Lastdevice
      ' zunchst Speicheradresse berechnen
      ' Szenenweises weiterspringen
      W2 = Scene - 1
      W2 = W2 * Lastdevice
      W2 = W2 * 4
      Address_ptr = 57 + W2
      ' Gerteweises weiterspringen
      W2 = W - 1
      W2 = W2 * 4 ' abzuspeichernde Kanalzahl (R,G,B,W=4)
      Address_ptr = Address_ptr + W2


      ' DMX Kanal berechnen
      W2 = W * Channelperdevice
      If W = Lastdevice Then
         W2 = W2 - 4
      Else
         W2 = W2 - 5
      End If

      ' Daten in EEPROM schreiben
      Writeeeprom Dmxfield(w2) , Address_ptr                ' Rot schreiben
      W2 = W2 + 1
      Address_ptr = Address_ptr + 1                         ' Adresspointer fr Grn um 1 erhhen
      Writeeeprom Dmxfield(w2) , Address_ptr                ' Grn schreiben
      W2 = W2 + 1
      Address_ptr = Address_ptr + 1                         ' Adresspointer fr Blau um 1 erhhen
      Writeeeprom Dmxfield(w2) , Address_ptr                ' Blau schreiben
      W2 = W2 + 1
      Address_ptr = Address_ptr + 1                         ' Adresspointer fr Weiss um 1 erhhen
      Writeeeprom Dmxfield(w2) , Address_ptr                ' Weiss schreiben
   Next
End Sub

Sub Setdmxch(byval Channel As Word , Byval Startvalue As Byte , Byval Endvalue As Byte , Byval Fadetime As Word)
   Local Steps As Integer
   Local Ftime As Long
   Local Ftemp As Single

   If Channel < 1 Or Channel > Lastchan Then Exit Sub

   If Fadetime = 0 Then
      ' Schalten
      Newdmxfield(channel) = Endvalue
      Dmxfield(channel) = Endvalue
   Else
      ' Dimmen
      Dmxfield(channel) = Startvalue
      ' Minimale Fadezeit ist 1ms/Schritt, also 255ms fr 0..255
      Steps = Startvalue - Endvalue
      Steps = Abs(steps)                                    ' Schritte, die es zu dimmen gilt
      Ftime = Fadetime * 1000
      Ftemp = Ftime / Steps
      Dmxsteps(channel) = Int(ftemp)                        ' Wartezyklen bis nchster Inkrementierungsschritt
      Newdmxfield(channel) = Endvalue
   End If
End Sub

Sub Stepupch(byval Channel As Word)
'   0..64..127..191..255
   Select Case Newdmxfield(channel)
      Case 0 To 63 : Call Setdmxch(channel , Dmxfield(channel) , 64 , 3)
      Case 64 To 126 : Call Setdmxch(channel , Dmxfield(channel) , 127 , 3)
      Case 127 To 190 : Call Setdmxch(channel , Dmxfield(channel) , 191 , 3)
      Case 191 To 255 : Call Setdmxch(channel , Dmxfield(channel) , 255 , 3)
   End Select
End Sub

Sub Stepdownch(byval Channel As Word)
'   0..64..127..191..255
   Select Case Newdmxfield(channel)
      Case 192 To 255 : Call Setdmxch(channel , Dmxfield(channel) , 191 , 3)
      Case 128 To 191 : Call Setdmxch(channel , Dmxfield(channel) , 127 , 3)
      Case 65 To 127 : Call Setdmxch(channel , Dmxfield(channel) , 64 , 3)
      Case 1 To 64 : Call Setdmxch(channel , Dmxfield(channel) , 0 , 3)
   End Select
End Sub

' ******************************************************************************
' * Interrupts und spezielle Funktionen                                        *
' ******************************************************************************

' DCF77-Interrupt
Sectic:
   ' Wird jede Sekunde ausgefhrt
   Set Sectick
Return

' TxD Complete Interrupt
Dmxsend:                                                    'Neues DMX512-Paket zusammenstellen
   If Dmxstate = 0 Then
      Dmxstate = 1
      Ubrr = 10                                             '90,9kBaud
'      $baud = 9090
      Udr = 0                                               'send Break
   Elseif Dmxstate = 1 Then
      Dmxstate = 2
      Curdmxch = 0
      Ubrr = 3                                              '250kBaud
'      $baud = 250000
      Udr = 0                                               'send Startbyte
   Else
      Curdmxch = Curdmxch + 1
      Waitus 10                                             'InterByteGap-Zeit - Standard ist 10s
      Udr = Dmxfield(curdmxch)                              'DMX-Wert des aktuellen Kanals senden
      If Curdmxch = Lastchan Then
         Dmxstate = 0                                       'neuer Break, wenn alle Kanle gesendet worden sind
      End If
   End If
Return

' Timer2 Interrupt
Dimmerkernel:                                               'Millisekundenintervall
   Timer2 = 130                                             ' Timer-Interrupt jede ms ausfhren (16000000MHz/128/125=1000kHz)

   For Bi = 1 To Lastchan
      If Dmxfield(bi) < Newdmxfield(bi) Then
         If Curdmxsteps(bi) < Dmxsteps(bi) Then
            Incr Curdmxsteps(bi)
         Else
            ' nach oben dimmen
            Curdmxsteps(bi) = 0
            Incr Dmxfield(bi)
         End If
      Elseif Dmxfield(bi) > Newdmxfield(bi) Then
         If Curdmxsteps(bi) < Dmxsteps(bi) Then
            Incr Curdmxsteps(bi)
         Else
            ' nach oben dimmen
            Curdmxsteps(bi) = 0
            Decr Dmxfield(bi)
         End If
      End If
   Next
Return



' ******************************************************************************
' * EEPROM-Daten                                                               *
' ******************************************************************************

' Abspeichern von Einstellungen und Tabellen
$eeprom
$eepromhex
Data 0                                                      ' erstes Byte ist nicht zu gebrauchen (siehe AppNote)
Data 8405% , 7996% , 7134% , 6025% , 4846% , 4202% , 4168% , 4790% , 5571% , 6367% , 7200% , 8003%       ' Zeiten fr Sonnenaufgang pro Monat
Data 16455% , 17150% , 18014% , 18848% , 19779% , 20469% , 20676% , 20154% , 19168% , 18016% , 16986% , 16356%       ' Zeiten fr Sonnenuntergang pro Monat
Data 000 , 015 , 060 , 000 , 000 , 000 , 000 , 000          ' Einstellungen (Ausschaltzeit Stunde, Ausschaltzeit Minute, Einblendzeit
Data 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000       ' Szene 1: Standard Orange
Data 255 , 000 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 223 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 223 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000       ' Szene 2: Rot, Orange, Gelb, Orange, Rot, Orange, Gelb, Orange
Data 255 , 177 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 074 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 074 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 074 , 000 , 000 , 255 , 177 , 000 , 000       ' Szene 3: HellOrange, Orange, DunkelOrange, Rot, Rot, DunkelOrange, Orange, HellOrange
Data 255 , 204 , 000 , 000 , 000 , 000 , 000 , 255 , 255 , 204 , 000 , 000 , 000 , 000 , 000 , 255 , 255 , 204 , 000 , 000 , 000 , 000 , 000 , 255 , 255 , 204 , 000 , 000 , 000 , 000 , 000 , 255 , 204 , 255 , 204 , 000       ' Szene 4: Gelb, Wei, Gelb, Wei, Gelb, Wei, Gelb, Wei
Data 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000 , 255 , 102 , 000 , 000       ' Szene 5: Standard Orange
Data 128 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 128 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 128 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 128 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000       ' Szene 6: Dunkelrot, Rot, Dunkelrot, Rot,  Dunkelrot, Rot,  Dunkelrot, Rot
Data 128 , 000 , 128 , 000 , 204 , 153 , 255 , 000 , 128 , 000 , 128 , 000 , 204 , 153 , 255 , 000 , 128 , 000 , 128 , 000 , 204 , 153 , 255 , 000 , 128 , 000 , 128 , 000 , 204 , 153 , 255 , 000 , 204 , 153 , 255 , 000       ' Szene 7: Dunkelviolett, Violett, Dunkelviolett, Violett, Dunkelviolett, Violett, Dunkelviolett, Violett
Data 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 204 , 255 , 204 , 000       ' Szene 8: Wei
Data 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000       ' Szene 9: Rot
Data 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000 , 000 , 000 , 255 , 000       ' Szene 10: Blau
Data 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000       ' Szene 11: Violett
Data 255 , 255 , 255 , 000 , 064 , 255 , 064 , 000 , 255 , 255 , 255 , 000 , 064 , 255 , 064 , 000 , 255 , 255 , 255 , 000 , 064 , 255 , 064 , 000 , 255 , 255 , 255 , 000 , 064 , 255 , 064 , 000 , 064 , 255 , 064 , 000       ' Szene 12: Wei, Grn, Wei, Grn, Wei, Grn, Wei, Grn
Data 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000 , 188 , 000 , 255 , 000       ' Szene 13: Violett
Data 255 , 000 , 000 , 000 , 255 , 255 , 255 , 000 , 255 , 000 , 000 , 000 , 255 , 255 , 255 , 000 , 255 , 000 , 000 , 000 , 255 , 255 , 255 , 000 , 255 , 000 , 000 , 000 , 255 , 255 , 255 , 000 , 255 , 255 , 255 , 000       ' Szene 14: Rot, Wei, Rot, Wei, Rot, Wei, Rot, Wei
Data 000 , 255 , 255 , 000 , 255 , 255 , 000 , 000 , 255 , 000 , 000 , 000 , 000 , 000 , 255 , 000 , 255 , 000 , 255 , 000 , 255 , 255 , 000 , 000 , 000 , 000 , 255 , 000 , 255 , 255 , 000 , 000 , 255 , 000 , 000 , 000       ' Szene 15: Trkis, Gelb, Rot, Blau, Violett, Gelb, Blau, Rot
Data 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000       ' Szene 16: Frei 1
Data 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000       ' Szene 17: Frei 2
Data 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000       ' Szene 18: Frei 3
Data 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000       ' Szene 19: Frei 4
Data 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000 , 000       ' Szene 20: Frei 5
$data

Wochentag:
Data "D?" , "Mo" , "Di" , "Mi" , "Do" , "Fr" , "Sa" , "So"